{$D+,E+,I+,L+,P-,Q-,R+,S+,T-,V+,X+,Y+}

PROGRAM Lover;
{* dynamic breath first algorithm for graph traversal *}
{* start from the last point and *}
{* find the ammount of fuel requied in each node *}
{* find the ammount of time to bring the required fuel *}
{* have to use list for edge represenation of graph *}
{* have to know recursive traversal for printing *}

CONST
  InFile  = 'lover.in';
  OutFile = 'lover.out';
  MAX = 3001 ; {* max number of nodes in the graph *}
  MMAX= 400 ; {* max number of edges for each node *}

TYPE
  plist = ^list;
  list = RECORD
           dest:integer;
           weight:integer;
           next:plist;
         END;
  pnode = ^node;
  node = RECORD
           id:integer;
           next:pnode;
         END;
VAR
  fin,fout:text;
  N : integer;         {* N = Number of nodes, 1 is starting, N is finish *}
  M : integer;         {* M = Number of edges in the graph *}
  K : integer;         {* K = Capacity of the vehicle *}

  e:ARRAY[0..MAX] of plist;  {* weights to travel *}
  P:array[0..MAX] of integer; {* prices per 1 litter fuel *}
  x:array[0..MAX] of integer;  {* next node for minimal path *}
  t:array[0..MAX] of integer;  {* minimal time required for each node *}

PROCEDURE InsertNode(i:integer;var p:pnode);
VAR
  q:pnode;
BEGIN
    {* insert at the begining of the list *}
    new(q);
    q^.id:=i;
    q^.next:=p;
    p:=q;
END;

PROCEDURE DeleteNode(var p:pnode);
VAR
  q:pnode;
BEGIN
    {* delete the first of the list *}
    q:=p;
    p:=p^.next;
    dispose(q);
END;

PROCEDURE InsertEdge(i,j,d:integer);
VAR
  q:plist;
BEGIN
    {* insert at the begining of the list *}
    {* insert distance d from i to j *}
    new(q);
    q^.dest:=j;
    q^.weight:=d;
    q^.next:=e[i];
    e[i]:=q;
END;

PROCEDURE DeleteEdge(i,j:integer);
VAR
  q:plist;
BEGIN
    {* insert at the begining of the list *}
    {* insert distance d from i to j *}
    q:=e[i];
    while q^.dest<>j do q:=q^.next;
    e[i]:=q;
END;

PROCEDURE Init;
VAR
  i:integer ;
  Nfrom,Nto,d : integer;
BEGIN
  {* initialize *}
  {* input file is *}
  {* the first row N, M, K *}
  {* next N-1 rows Pi prices *}
  {* next M rows From, To, Dist *}

  assign(fin,InFile);
  reset(fin);
  readln(fin,N,M,K);
  for i:=1 to M do begin
    readln(fin,Nfrom,Nto,d);
    if d<=K then InsertEdge(Nfrom,Nto,d);
    if d<=K then InsertEdge(Nto,Nfrom,d);
  end;
  for i:=1 to N-1 do readln(fin,P[i]);
  close(fin);

  for i:=1 to N do t[i]:=MaxInt;
{  for i:=1 to N do req[i]:=MaxInt;}
END; {* Init *}

PROCEDURE Done;
VAR
{  q:plist;}
  i,ii,j:integer;
  howmany:LongInt;
  temp:integer;
  b,c,y:array[1..MAX] of integer;
  MoreFuel:integer;
{  TakeFuel,StartFuel,DestFuel,ReturnFuel:integer;}

BEGIN
  assign(fout,OutFile);
  rewrite(fout);
  write(fout,t[N]);

  {* the shortest route is found *}
  {* now make changes *}
  {* N - number of points in the shortest route *}
  {* x[i] - the i-th point on the shortest route *}

  t[1]:=N;
  i:=1;
  repeat
    inc(i);
    t[i]:=x[t[i-1]];
  until t[i]=1;
  N:=i;
  for i:=1 to N do x[i]:=t[N-i+1];


  {* now delete all other paths in the list *}
  for i:=1 to N-1 do DeleteEdge(x[i],x[i+1]);
  e[x[N]]^.weight:=0;

  {* route x[1],x[2],...,x[N] *}
  {* fuel sells P[x[1]],P[x[2],... *}
  {* distances are e[x[i]]^.weight *}

  {* simple strategy *}
{
  howmany:=0;
  for i:=1 to N-1 do begin
    t[i]:=e[x[i]]^.weight;
    howmany:=howmany+t[i]*P[x[i]];
  end;
}
    {* greedy strategy *}
    {* take max fuel if cheaper *}
    {* sort the array *}
    {* y = index to the cheapest fuel *}
  for i:=1 to N do y[i]:=i;
  for i:=1 to N-2 do
    for j:=i+1 to N-1 do begin
      if P[x[y[i]]]>P[x[y[j]]] then begin
        temp:=y[i];
        y[i]:=y[j];
        y[j]:=temp
      end;
  end;

  {* t = ammount of fuel required total in nodes! *}
  for i:=1 to N-1 do b[i]:=0;
  for i:=1 to N-1 do c[i]:=0;
  t[N]:=0;
  for i:=N downto 2 do t[i-1]:=t[i]+e[x[i-1]]^.weight;

  for i:=1 to N-1 do begin
    {* is there need to buy *}
    if (t[y[i]]>0) and (t[y[i]]>c[y[i]]) then begin
      {* buy max possible ammount *}
      if (t[y[i]]<K)
        then b[y[i]]:=t[y[i]]-c[y[i]]
        else b[y[i]]:=K-c[y[i]];
      c[y[i]]:=c[y[i]]+b[y[i]];
      MoreFuel:=c[y[i]]-e[x[y[i]]]^.weight;
      {* take this ammount to right *}
      ii:=1;
      while (MoreFuel>0) and (y[i]+ii<=N) do begin
        c[y[i]+ii]:={c[y[i]+ii]+}MoreFuel;
        MoreFuel:=c[y[i]+ii]-e[x[y[i]+ii]]^.weight;
        inc(ii)
      end;
      {* eliminate the need to the left *}
      if (c[y[i]]=K) or (c[y[i]]=t[y[i]]) then begin
        ii:=0;
        temp:=t[y[i]+ii];
        while (y[i]+ii>0) and (t[y[i]+ii]>0) do begin
          t[y[i]+ii]:=t[y[i]+ii]-temp;
          if t[y[i]+ii]<0 then t[y[i]+ii]:=0;
          dec(ii);
        end;
      end;
    end;
  end;

  howmany:=0;
  for i:=1 to N-1 do begin
    howmany:=howmany+b[i]*P[x[i]];
  end;

{
  writeln(fout,N:5,'  ',howmany);
  for i:=1 to N do writeln(fout,i:3,' ',x[i]:3,' ',P[x[i]]:3,' ',b[i]:3,' ',e[x[i]]^.weight:3);
}
  writeln(fout,' ',howmany);
  close(fout);
END;

PROCEDURE Run;
VAR
    pnodes,qnodes : pnode; {* required for Breath First Algorithm *}
    ii,totaltime:integer;

  PROCEDURE Dynamic(i:integer);
  VAR
     j,d:integer;
     q,p:plist;

  BEGIN
    q:=e[i];
    while q<>nil do begin
      {* this is one of the succersors *}
      j:=q^.dest;
      d:=q^.weight;

      {* fill the dynamic value *}
      totaltime:= d + t[i];
      if t[j]>totaltime then begin
        t[j]:=totaltime;
        x[j]:=i;

        {* one more node for new wave Breath First Algorithm *}
        InsertNode(j,qnodes)
      end;

      {* go to next edge *}
      q:=q^.next
    end;
  END; {* Dynamic *}

BEGIN
  {* no nodes in the route *}
  qnodes:=nil;
  InsertNode(1,qnodes);
  t[1]:=0;
  repeat
    pnodes:=qnodes;
    qnodes:=nil;
    while pnodes<>nil do begin
      Dynamic(pnodes^.id);
      DeleteNode(pnodes)
    end;
  until qnodes=nil;
END;

BEGIN
  Init;
  Run;
  Done
END.
